VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "HTMLClass"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
Option Explicit
'---------------------------------------------------------
' Source: HTML.cls
' Author: Mike Miller
' Reason: HTML Rendering engine, see HTML.TEST
' for an example of one way to use this class.
'---------------------------------------------------------
' Notes:
' 1) Certain methods support optional data so that a corresponding
'    call to method_end is not needed (if optional data is passed).
'    Examples:
'    HTML.H  level, HeadingString
'    HTML.P  ParagraphString
'    HTML.TD TableDataString
'    if param ismissing then H_end, P_end, and TD_end can be called.
'-------------------------------------------------------------------
Public zHtml          As String ' HTML result string
Public inxHTML        As Long   ' inxHtml is index in zHtml
Public zText          As String ' Plain Text result string
Public inxText        As Long   ' inxText is index in zText

' General properties
Public Align          As String ' left, right, center Table, TD, P

' <Hn> properties
Public Hlevel         As Long   ' Heading 1, 2, ...

' Font Properties
Public FontColor      As String ' font color=#"rrggbb" in hex
Public FontFace       As String ' Arial, Courier, ...
Public FontSize       As String ' 1, 2, 3, ... 7

' Table Properties
Public BGcolor        As String  ' BGcolor=rrggbb in hex
Public Border         As String  ' border=<width> in pixels
Public BorderCollapse As Boolean ' style="border-collapse: collapse"
Public CellPadding    As String  ' CellPadding=1
Public CellSpacing    As String  ' CellSpacing=2
Public Cols           As String  ' table column count, cols=10
Public Width          As String  ' HTML width=100%
Public Valign         As String  ' valign=top, middle, bottom, baseline
Public ZeroSuppress   As Boolean ' true => don't show zeros

Private TH_flag       As Boolean ' makes a TD call be a TH call
Private zeros         As String ' string of all zeros, see class init

Public Sub A(href As String, htxt As String)
'-------------------------------------------
' Creates tag <a href="href">htxt</a>
'------------------------------------------
  On Error Resume Next
  ConCatTEXT htxt
  ConCatHTML "<a href=" & """" & href & """" & ">" & htxt & "</a>"
End Sub

Public Sub add(txt As String)
'--------------------------------
' Adds plain text to out stream
'--------------------------------
  On Error Resume Next
  ConCatTEXT txt
  ConCatHTML txt
End Sub

Public Sub b() ' bold
'-------------
' <b>
'-------------
  On Error Resume Next
  ' no emit for plain text
  ConCatHTML "<b>"
End Sub
Public Sub B_end()
  On Error Resume Next
  ' no emit for plain text
  ConCatHTML "</b>"
End Sub

Public Sub Banner(txt As String, Optional txt2)
  On Error Resume Next
  ' NonStop ASAP Banner
  ' Plain text
  NewLine
  ConCatTEXT txt
  If Not IsMissing(txt2) Then ConCatTEXT " - " & txt2
  
  ' HTML
  Cols = 1
  Align = ""
  Border = 0
  BGcolor = "000050"
  CellPadding = 0
  CellSpacing = 0
  Width = "100%"
  Table
    ClearSettings
    TR
      Align = "left"
      TD
        Align = ""
        Width = ""
        FontSize = "2"
        FontColor = "FFFFFF"
        FontFace = "Arial"
        Font
          b
          ConCatHTML "&nbsp; " & txt
          If Not IsMissing(txt2) Then ConCatHTML "&nbsp" & txt2
          B_end
        Font_end
      TD_end
    TR_end
  Table_end
End Sub
Public Sub BR()
'--------------
' emits line break
'-----------------
  ' plain text
  NewLineText
  
  ' HTML
  ConCatHTML "<br>"
  NewLineHTML
End Sub
Public Sub Clear()
'-----------------
' Clears everything, eg: settings, out strings, and indexes
'----------------------------------------------------------
  On Error Resume Next
  inxHTML = 0
  inxText = 0
  zHtml = ""
  zText = ""
  ClearSettings
End Sub

Public Sub ClearSettings()
'-------------------------
' Clears HTML setttings and variables, but does not
' touch output strings or indexes into strings.
'-------------------------------------------------
  On Error Resume Next
 
  ' general
  Align = ""
  
  ' <h1> <h2> ...
  Hlevel = 1
  
  ' font
  FontColor = ""
  FontFace = ""
  FontSize = ""
  
  ' table
  BGcolor = ""
  Border = ""
  CellPadding = ""
  CellSpacing = ""
  Cols = ""
  Width = ""
  Valign = ""
  ZeroSuppress = False
End Sub

Public Sub Comment(Optional Data)
  On Error Resume Next
  ' no emit for plain text
  
  ' HTML
  NewLineHTML
  ConCatHTML "<!--"
  If Not IsMissing(Data) Then
     ConCatHTML CStr(Data)
     Comment_end
  End If
End Sub
Public Sub Comment_end()
  On Error Resume Next
  ' no emit for plain text
  
  ' HTML
  ConCatHTML "-->" & vbCrLf
End Sub

Public Sub ConCatHTML(Html As String)
  ConCat zHtml, inxHTML, Html
End Sub

Public Sub ConCatTEXT(txt As String)
  ConCat zText, inxText, txt
End Sub

Public Sub ConCat(oldS$, inx&, newS$, Optional ChunkBytes_)
'---------------------------------------------------------- 97/3/24
' This proc improves concatenation
' performance by doing chunk allocs, and
' using Mid instead of concatenation.
'
' Add string newS to oldS at index=inx.
' Note len(oldS) checked to see if room.
' If not, oldS is grown by ChunkBytes.
' Notes: inx ranges from 1.. upward
' Caller decides if trim needed when done.
' 5/12/00 add optional ChunkBytes for large chunk perf
'------------------------------------------------------
  Dim Chunks&            ' number of chunks to allocate
  Dim ChunkBytes As Long ' size of chunk 5/12/00
  Const ChunkBytesDefault As Long = 1000
  On Error Resume Next
  
  If Not IsMissing(ChunkBytes) Then ' 5/12/00 allow user to specify
     ChunkBytes = CLng(ChunkBytes_)
  Else
     ChunkBytes = ChunkBytesDefault
  End If
  
  ' See if we need to allocate space, and
  ' if so how many chunks of space to allocate
  If Len(oldS) < (inx + Len(newS) + 10) Then
    If Len(oldS) <= 0 Then inx = 1 ' 97/3/31 sanity ck
    If inx <= 0 Then inx = 1
    Chunks = 1 + (Len(newS) / ChunkBytes)
    oldS = oldS & Space(Chunks * ChunkBytes)
  End If

  Mid(oldS, inx) = newS
  inx = inx + Len(newS)

End Sub

Public Property Let BGcolorRGB(RGBv As Long)
'-------------------------------------------
' converts RGBv value to hex string and stores in BGcolor attr
'-------------------------------------------------------------
  Dim s As String
  Dim R As Long
  Dim g As Long
  Dim b As Long
  Dim Rs As String
  Dim Gs As String
  Dim Bs As String
  On Error Resume Next
  
  R = RGBv Mod 256
  Rs = Hex(R)
  If Len(Rs) = 1 Then Rs = "0" & Rs
  
  g = Int(RGBv / 256) Mod 256
  Gs = Hex(g)
  If Len(Gs) = 1 Then Gs = "0" & Gs
  
  b = Int(Int(RGBv / 256) / 256) Mod 256
  Bs = Hex(b)
  If Len(Bs) = 1 Then Bs = "0" & Bs
  
  s = Rs & Gs & Bs
  BGcolor = s
End Property

Public Function HexToLong(HexStr As String) As Long
'--------------------------------------------------
' Convert Hex string 0123456789ABCDEF to long
'----------------------------------------------
  Dim HexTable As String
  Dim i As Long
  Dim s As String
  Dim z As String
  Dim HexValue As Long
  HexTable = "0123456789ABCDEF"
  HexToLong = 0
  HexValue = 0
  s = UCase(HexStr)
  While Len(s) > 0
    z = Mid(s, 1, 1)
    If Len(s) < 2 Then s = "" Else s = Mid(s, 2) ' trim first char from s
    i = InStr(HexTable, z)
    If i <= 0 Then Exit Function
    i = i - 1
    HexValue = HexValue * 16 + i
  Wend
  HexToLong = HexValue
End Function

Public Function BGColorDark() As Boolean
'---------------------------------------
' Analyzes BGcolor string to see if its too dark for black font.
' if so return true
'---------------------------------------------------------------
  Dim s As String
  Dim R As Long
  Dim g As Long
  Dim b As Long
  Dim Rs As String
  Dim Gs As String
  Dim Bs As String
  On Error Resume Next
  
  BGColorDark = False
  If Len(BGcolor) <> 6 Then Exit Function
  
  Rs = Mid(BGcolor, 1, 2)
  Gs = Mid(BGcolor, 3, 2)
  Bs = Mid(BGcolor, 5, 2)
  
  If HexToLong(Rs) < 160 And HexToLong(Gs) < 160 And HexToLong(Bs) < 160 Then
     BGColorDark = True
  End If
  
  If HexToLong(Rs) = 0 And HexToLong(Gs) = 0 And HexToLong(Bs) <= 255 Then
     BGColorDark = True
  End If
  
  Exit Function
End Function

Public Sub Body()
'----------------
' Start of HTML body
'-------------------
  NewLineHTML
  ConCatHTML "<body>"
End Sub
Public Sub Body_end()
'--------------------
' end HTML body
'-------------------
  NewLineHTML
  ConCatHTML "</body>"
End Sub

Public Sub crHTML()
'------------------
' Forces crlf to stream
'-----------------------
  ConCatHTML vbCrLf
End Sub
Public Sub crTEXT()
'------------------
' Forces crlf to stream
'-----------------------
  ConCatTEXT vbCrLf
End Sub

Public Sub cr()
'--------------
' Forces crlf to stream
'-----------------------
  On Error Resume Next
  crHTML
  crTEXT
End Sub

Sub File_Save(SaveName$, SaveString$, confirm As Boolean, Dialog As CommonDialog, Optional AppEnd)
'-------------------------------------------------------------------------------------------------
' Saves: SaveString to SaveName.
' If SaveString is too big, multiple writes are done.
' If confirm, do "Save As", if file exists, ask ok?
' Note: this proc handles infinitely large strings
' 2004/4/13 - if Append then add SaveString to end of file SaveName
'------------------------------------------------------------------
  Dim fileno As Integer
  Dim savetitle As String
  Dim saveAs As Integer ' true if its a save as
  Dim answer As Integer
  Dim z As String
  Dim s As String
  Dim Done As Boolean
  Dim ioInx As Long
  Dim ioLen As Long
  Dim ioBuf As String
  Dim ioByteOff As Long ' 2004/4/13 file byte offset position
  
  On Error Resume Next
  
  If confirm Then 'save as
    Dialog.DefaultExt = "TXT"
    s = SaveName
'    z = file_title(s, True)
    z = s
    Dialog.filename = z
    ' determine init path
    If (Len(s) - Len(z)) <= 1 Then
      z = ""
    Else
      z = Left(s, Len(s) - Len(z) - 1)
    End If
    If z <> "" Then Dialog.InitDir = z
    Dialog.DialogTitle = "Save file text as <filename>?"
    Dialog.Filter = "Text or EDL Files (*.TXT;*.EDL)|*.TXT;*.EDL|All Files (*.*)|*.*"
    Dialog.FilterIndex = 1
    Dialog.CancelError = True
    Dialog.Action = 2
    If Err.Number <> 0 Then
      If Err = cdlCancel Then Exit Sub 'just exit, cancel save
      MsgBox "File NOT saved, err=" & Err & Chr(10) & Error$, vbExclamation
      Exit Sub
    End If
    SaveName = Dialog.filename
    savetitle = Dialog.FileTitle
    'Unload dialog
  End If
  
  ' Check for Confirm
  On Error Resume Next
  answer = GetAttr(SaveName)
  If confirm And (Err = 0) Then
    'file exists, check with them!
    answer = MsgBox(UCase(SaveName) & vbCrLf & "File aready exists!" & vbCrLf & vbCrLf & "Replace existing File?", vbExclamation + vbYesNo)
    If answer = vbNo Then Exit Sub
  End If
  
  ' Purge old file
  If IsMissing(AppEnd) Then ' 2004/4/13
     Kill SaveName
  End If
  
  ' Save the file
  fileno = FreeFile
  Err.Clear
  Open SaveName For Binary As fileno Len = 32767
  If Err.Number <> 0 Then
    Close fileno
    s = "File " & SaveName & vbCrLf & vbCrLf & "NOT saved, error = " & Err.Number & vbCrLf & Err.Description
    MsgBox s, vbExclamation
    Exit Sub
  End If

  ioInx = 1     ' index in ioBuf
  ioByteOff = 1 ' index in file
  If Not IsMissing(AppEnd) Then ' if append, start at eof
     ioByteOff = LOF(fileno) + 1
  End If
  
  Do 'handle multiple blocks if greater than max io size
    ioLen = Len(SaveString) - ioInx + 1
    If ioLen > 32767 Then
      ioLen = 32767
      Done = False
    Else
      Done = True
    End If
    ioBuf = Mid(SaveString, ioInx, ioLen)
    Err.Clear
    Put fileno, ioByteOff, ioBuf  ' 2004/4/13 allow optional append
    ioByteOff = ioByteOff + ioLen ' 2004/4/13 append index in file
    ioInx = ioInx + ioLen
    If Err.Number <> 0 Then
      s = "File " & SaveName & vbCrLf & vbCrLf & "NOT SAVED, ERROR = " & Err.Number & vbCrLf & vbCrLf & UCase(Err.Description)
      MsgBox s, vbExclamation
      Exit Sub
    End If
  Loop Until Done
  
  Close fileno
  
End Sub

Public Sub Font()
'----------------
' <font color=#"rrggbb" face="Arial" size="2">
'---------------------------------------------
  Dim HexColor As String
  Dim s As String
  On Error Resume Next
  ' no emit plain text
    
  ' HTML
  ConCatHTML "<font"
  
  If FontColor <> "" Then ' color specified
     HexColor = FontColor
     If Len(HexColor) < 6 Then HexColor = Mid(zeros, 1, 6 - Len(HexColor))
     s = " color=" & """" & "#" & HexColor & """"
     ConCatHTML s
  End If
  
  If FontFace <> "" Then
     ConCatHTML " face=" & """" & FontFace & """"
  End If
  
  If FontSize <> "" Then
     ConCatHTML " size=" & """" & FontSize & """"
  End If
  
  ConCatHTML ">"
  
End Sub
Public Sub Font_end()
  On Error Resume Next
  ' No emit for plain text
  
  ' HTML
  ConCatHTML "</font>"
  FontSize = ""
  FontColor = ""
  FontFace = ""
End Sub

Public Sub H(level As Long, Optional Data)
'-----------------------------------------
' <h1 align="..."> if data emit data & </Hn>
'-------------------------------------------
  On Error Resume Next
  NewLine
  
  ' Text only
  If Not IsMissing(Data) Then ConCatTEXT CStr(Data)
  
  ' HTML
  ConCatHTML "<h" & level
  Hlevel = level
  If Align <> "" Then
     ConCatHTML " align=" & """" & Align & """"
  End If
  ConCatHTML ">"
  If Not IsMissing(Data) Then
     ConCatHTML CStr(Data)
     H_end
  End If
End Sub
Public Sub H_end()
'----------------
' </h1>
'----------------
  On Error Resume Next
  ' No emit for plain text
  
  ' HTML
  ConCatHTML "</h" & Hlevel & ">" & vbCrLf
End Sub

Public Sub Head()
  On Error Resume Next
  ' No emit for plain text
  
  ' HTML
  ConCatHTML "<head>"
End Sub
Public Sub Head_end()
  On Error Resume Next
  ' No emit for plain text
  
  ' HTML
  ConCatHTML "</head>"
End Sub

Public Sub Html()
  On Error Resume Next
  ' No emit for plain text
  
  ' HTML
  ConCatHTML "<html>"
End Sub
Public Sub HTML_end()
  On Error Resume Next
  ' No emit for plain text
  
  ' HTML
  ConCatHTML "</html>"
  
  ' Strip trailing blanks from buffer
  zText = RTrim(zText)
  inxText = Len(zText) + 1
  zHtml = RTrim(zHtml)
  inxHTML = Len(zHtml) + 1
End Sub

Public Sub Meta(Optional RefreshSecs, Optional Keywords)
'-------------------------------------------------------
' RefreshSecs => <Meta HTTP-EQUIV=refresh CONTENT=RefreshSecs>
' Keywords    => <Meta HTTP-EQUIV="Keywords">
'-------------------------------------------------------------
  On Error Resume Next
  ' No emit for plain text
  
  ' HTML
  If Not IsMissing(RefreshSecs) Then
     If RefreshSecs > 0 Then
        NewLineHTML
        ConCatHTML "<Meta HTTP-EQUIV=" & """" & "refresh" & """" & " content=" & """" & RefreshSecs & """" & ">"
     End If
  End If
  If Not IsMissing(Keywords) Then
     NewLineHTML
     ConCatHTML "<Meta HTTP-EQUIV=" & """" & Keywords & """" & " content=" & """" & Keywords & """" & ">"
  End If
End Sub

Public Sub NewLineHTML()
'-----------------------
' Make sure zText is at a newline
' that is the zText buffer ends in cr lf
'---------------------------------------
  If inxHTML < 3 Then Exit Sub
  If Mid(zHtml, inxHTML - 2, 2) = vbCrLf Then Exit Sub
  ConCatHTML vbCrLf
End Sub

Public Sub NewLineText()
'-----------------------
' Make sure zText is at a newline
' that is the zText buffer ends in cr lf
'---------------------------------------
  If inxText < 3 Then Exit Sub
  If Mid(zText, inxText - 2, 2) = vbCrLf Then Exit Sub
  ConCatTEXT vbCrLf
End Sub

Public Sub NewLine()
  NewLineHTML
  NewLineText
End Sub

Public Sub P(Optional Data)
'---------------------------
' paragraph <p align="..."> if DATA then emit Data and </p>
'----------------------------------------------------------
  On Error Resume Next
  NewLine
  
  ' plain text
  If Not IsMissing(Data) Then ConCatTEXT CStr(Data)
  
  ' Html
  ConCatHTML "<p"
  If Align <> "" Then
     ConCatHTML " align=" & """" & Align & """"
  End If
  ConCatHTML ">"
  If Not IsMissing(Data) Then
     ConCatHTML CStr(Data)
     P_end
  End If
End Sub
Public Sub P_end()
'----------------
' </h1>
'----------------
  On Error Resume Next
  ' No emit for plain text
  
  ' HTML
  ConCatHTML "</p>"
End Sub

Public Sub Style()
'-----------------
' Emits embedded style sheet
'---------------------------
  On Error Resume Next
  ' No emit for plain text
  
  ' HTML
  NewLineHTML
  ConCatHTML "<style>" & vbCrLf
  
  ConCatHTML "BODY {font-family: Verdana, Arial, Geneva, Helvetica, sans-serif;font-size: 10pt;}" & vbCrLf
  ConCatHTML "P    {font-size: 9pt}" & vbCrLf
  ConCatHTML "TABLE{font-size: 10pt; text-align: center; font-family: Verdana, Arial, sans-serif;}" & vbCrLf
  
  ConCatHTML "</style>" & vbCrLf

End Sub

Public Sub Table()
'-----------------
' <table
'  BGColor="#FF0000"
'  border="1"
'  Cellpadding="1"
'  Cellspacing="1"
'  cols="10"
'  width="100%">
'-------------------------------------
  On Error Resume Next
  ' No emit for plain text
  
  ' HTML
  NewLineHTML
  ConCatHTML "<table"
  If Align <> "" Then
     ConCatHTML " align=" & """" & Align & """"
  End If
  If BGcolor <> "" Then
     ConCatHTML " bgcolor=" & """" & "#" & BGcolor & """"
  End If
  If Border <> "" Then
     ConCatHTML " border=" & """" & Border & """"
  End If
  If CellPadding <> "" Then
     ConCatHTML " cellpadding=" & """" & CellPadding & """"
  End If
  If CellSpacing <> "" Then
     ConCatHTML " cellspacing=" & """" & CellSpacing & """"
  End If
  If Cols <> "" Then
     ConCatHTML " cols=" & """" & Cols & """"
  End If
  If Width <> "" Then
     ConCatHTML " width=" & """" & Width & """"
  End If
  If BorderCollapse Then
   ' style="border-collapse: collapse" bordercolor="#111111"
     ConCatHTML " style=" & """" & "border-collapse: collapse" & """" & " bordercolor=" & """" & "#111111" & """"
  End If
  Align = ""
  BGcolor = ""
  Width = ""
  ConCatHTML ">"
End Sub
Public Sub Table_end()
  On Error Resume Next
  NewLine
  ConCatHTML "</table>"
  NewLineHTML
End Sub

Public Sub Test(HtmlStr As String, TextStr As String)
  Dim H As HTMLClass
  Dim i As Long
  Dim j As Long
  Dim Heading As String

  Set H = New HTMLClass
  H.Clear
  H.Html
  H.Head
  H.Meta RefreshSecs:=10
  H.Title "Nonstop ASAP email alert"
  H.Style
  H.Head_end
    
  H.Banner "Nonstop ASAP", "Report"
  H.ClearSettings
  H.H 3, "ASAP Object State Summary"
  
  H.Cols = 10
  H.Border = 3
  H.CellPadding = 1
  H.CellSpacing = 0
  H.Width = "100%"
  H.Table
  
  For i = 1 To 10
    H.TR
    For j = 1 To 10
      If i = 1 Then H.BGcolorRGB = (QBColor(5 + j))
      H.TD "" & i & "," & j
    Next j
    H.BGcolor = ""
    H.TR_end
  Next i
  H.Table_end
  
  H.ClearSettings
  H.cr
  H.H 3, "State Change Detail"
  H.Cols = 4
  H.Align = "center"
  H.Border = 3
  H.CellPadding = 1
  H.CellSpacing = 0
  H.Width = "100%"
  H.Table
    H.BGcolor = "C0C0C0"
    H.Align = "left"
    H.TR
'      H.TD "Time", "20%", Bold:=True, pWidthText:=18, pAlignText:="left"
'      H.TD "Status", "10%", Bold:=True, pWidthText:=10, pAlignText:="left"
'      H.TD "Domain", "40%", Bold:=True, pWidthText:=25, pAlignText:="left"
'      H.TD "Cause", "30%", Bold:=True, pWidthText:=10, pAlignText:="left"
      H.TH "Time", "20%", pWidthText:=18, pAlignText:="left"
      H.TH "Status", "10%", pWidthText:=10, pAlignText:="left"
      H.TH "Domain", "40%", pWidthText:=25, pAlignText:="left"
      H.TH "Cause", "30%", pWidthText:=10, pAlignText:="left"
    H.TR_end
    H.Align = "left"
    H.TR
      H.TD "04-19-2004 07:20", pWidthText:=18, pAlignText:="left"
      H.TD "Down", RGBv:=RGB(255, 0, 0), pWidthText:=10, pAlignText:="left"
      H.TD "\Chicago\Cpu\02", pWidthText:=25, pAlignText:="left"
      H.TD "State Down", pWidthText:=10, pAlignText:="left"
    H.TR_end
    H.Align = "left"
    H.TR
      H.TD "04-19-2004 05:03", pWidthText:=18, pAlignText:="left"
      H.TD "Warning", RGBv:=RGB(255, 255, 0), pWidthText:=10, pAlignText:="left"
      H.TD "\Chicago\Disk\$Data", pWidthText:=25, pAlignText:="left"
      H.TD "State Mirror", pWidthText:=10, pAlignText:="left"
    H.TR_end
  H.Table_end
  
  H.ClearSettings
  H.cr
  H.P
    H.FontSize = 1
    H.FontFace = "arial"
    H.Font
      H.add "Questions - "
      H.A "mailto:support@nonstopasap.com", "Support@NonstopAsap.com"
      H.BR
      H.add "Last mod - " & Format(Now, "mmm d yyyy - hh:mm:ss")
    H.Font_end
  H.P_end
  
  H.HTML_end
  
  HtmlStr = Trim(H.zHtml)
  TextStr = Trim(H.zText)
  Set H = Nothing
  
End Sub

Public Sub Title(txt As String)
  On Error Resume Next
  ' No emit for plain text
  
  ' HTML
  NewLineHTML
  ConCatHTML "<title>" & txt & "</title>" & vbCrLf
End Sub

Public Sub TR()
  On Error Resume Next
  ' Plain Text
  NewLineText
  
  ' HTML
  NewLineHTML
  ConCatHTML "<tr"
  If Align <> "" Then
     ConCatHTML " align=" & """" & Align & """"
  End If
  If BGcolor <> "" Then
     ConCatHTML " bgcolor=" & """" & "#" & BGcolor & """"
  End If
  If Valign <> "" Then
     ConCatHTML " valign=" & """" & Valign & """"
  End If
  ConCatHTML ">"
  
  Align = ""
  BGcolor = ""
  Valign = ""

End Sub
Public Sub TR_end()
  On Error Resume Next
  ' No emit for plain text
  
  ' HTML
  ConCatHTML "</tr>" & vbCrLf
End Sub

Public Sub TH(Optional Data, Optional pWidth, Optional pWidthText, Optional RGBv, Optional pAlign, Optional pAlignText, Optional Bold)
  TH_flag = True
  TD Data, pWidth, pWidthText, RGBv, pAlign, pAlignText, Bold
  TH_flag = False
End Sub

Public Sub TD(Optional Data, Optional pWidth, Optional pWidthText, Optional RGBv, Optional pAlign, Optional pAlignText, Optional Bold)
'-------------------------------------------------------------------------------------------------------------------------------------
' Table data, if DATA then emit Data and </TD>
' Note RGBv is expected to be a windows RGB long (red rightmost byte)
'--------------------------------------------------------------------
  Dim s         As String
  Dim lb        As Long    ' leading blanks
  Dim W         As Long    ' width chars
  Dim cell      As String  ' cell value
  Dim Jsfy      As String  ' align left, right, center
  Dim FontWhite As Boolean ' background color too dark for dark fontcolor
  Dim HideZeros As Boolean ' true implies don't show cell with value of 0
  Dim i         As Long
  On Error Resume Next
  
  ' Plain Text
  W = 10             ' default cell width
  If Not IsMissing(pWidthText) Then
     W = pWidthText
     If Err.Number Then W = 10
  End If
  If IsMissing(Data) Then cell = " " Else cell = Data
  If ZeroSuppress Then ' check whether to suppress display of 0 value
   If IsNumeric(cell) Then
      i = CLng(cell)
      If i = 0 Then cell = " "
   End If
  End If
  Jsfy = Align
  If Not IsMissing(pAlign) Then Jsfy = pAlign
  If Not IsMissing(pAlignText) Then Jsfy = pAlignText
  Select Case UCase(Jsfy)
  Case "RIGHT", ""
     If Len(cell) < W Then cell = Space(W - Len(cell)) & cell
  Case "LEFT"
     If Len(cell) < W Then cell = cell & Space(W - Len(cell))
  Case Else
  End Select
  If Len(cell) > W Then cell = cell & " "
  ConCatTEXT cell
  
  
  ' HTML
  If TH_flag Then
     ConCatHTML "<th"
  Else
     ConCatHTML "<td"
  End If
  
  s = Align
  If Not IsMissing(pAlign) Then s = pAlign
  If s <> "" Then
     ConCatHTML " align=" & """" & s & """"
  End If
  
  If Not IsMissing(RGBv) Then BGcolorRGB = CLng(RGBv) ' sets BGcolor
  s = BGcolor
  If s <> "" Then
     ConCatHTML " bgcolor=" & """" & "#" & s & """"
  End If
  
  If Valign <> "" Then
     ConCatHTML " valign=" & """" & Valign & """"
  End If
  
  s = Width
  If Not IsMissing(pWidth) Then s = pWidth
  If s <> "" Then
     ConCatHTML " width=" & """" & s & """"
  End If
  ConCatHTML ">"
  
  If Not IsMissing(Data) Then
     FontWhite = BGColorDark
     If FontWhite Then
        FontColor = "FFFFFF"
        Font
     End If
     If Not IsMissing(Bold) Then b
     If ZeroSuppress Then ' check whether to suppress display of 0 value
        If IsNumeric(Data) Then
           i = CLng(Data)
           If i = 0 Then HideZeros = True
        End If
     End If
     If Not HideZeros Then ConCatHTML CStr(Data)
     If Not IsMissing(Bold) Then B_end
     If FontWhite Then Font_end
     ' TD_end -> Note </TD> is not required by HTML
  End If
  
  Align = ""
  BGcolor = ""
  Valign = ""
  Width = ""
  
End Sub
Public Sub TD_end()
  On Error Resume Next
  ' No emit for plain text
  
  ' HTML
  ConCatHTML "</td>"
End Sub

Public Sub TD_many(Optional t1, Optional t2, Optional t3, Optional t4, Optional t5, Optional t6, Optional t7, Optional t8, Optional t9, Optional t10)
'-----------------------------------
' creates series of <td>t1</td> tags
'-----------------------------------
  If Not IsMissing(t1) Then TD t1
  If Not IsMissing(t2) Then TD t2
  If Not IsMissing(t3) Then TD t3
  If Not IsMissing(t4) Then TD t4
  If Not IsMissing(t5) Then TD t5
  If Not IsMissing(t6) Then TD t6
  If Not IsMissing(t7) Then TD t7
  If Not IsMissing(t8) Then TD t8
  If Not IsMissing(t9) Then TD t9
  If Not IsMissing(t10) Then TD t10
End Sub
Private Sub Class_Initialize()
  On Error Resume Next
  Dim i As Long
  zeros = Space(80)
  For i = 1 To 80
    Mid(zeros, i) = "0"
  Next i
  inxHTML = 0
  inxText = 0
  zHtml = ""
  zText = ""
  FontColor = -1
  BorderCollapse = True
End Sub
